==> maven-223.8836.41-sources.jar: org/jetbrains/idea/maven/plugins/groovy/wizard/MavenGroovyNewProjectWizard.kt <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package org.jetbrains.idea.maven.plugins.groovy.wizard

import com.intellij.framework.library.FrameworkLibraryVersion
import com.intellij.ide.projectWizard.NewProjectWizardCollector.BuildSystem.logAddSampleCodeChanged
import com.intellij.ide.projectWizard.NewProjectWizardConstants.BuildSystem.MAVEN
import com.intellij.ide.projectWizard.generators.AssetsNewProjectWizardStep
import com.intellij.ide.starters.local.StandardAssetsProvider
import com.intellij.ide.wizard.GitNewProjectWizardData.Companion.gitData
import com.intellij.ide.wizard.NewProjectWizardBaseData.Companion.name
import com.intellij.ide.wizard.NewProjectWizardBaseData.Companion.path
import com.intellij.ide.wizard.NewProjectWizardStep
import com.intellij.ide.wizard.chain
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.observable.properties.ObservableMutableProperty
import com.intellij.openapi.observable.util.bindBooleanStorage
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.ui.distribution.DistributionInfo
import com.intellij.openapi.ui.ValidationInfo
import com.intellij.ui.CollectionComboBoxModel
import com.intellij.ui.UIBundle
import com.intellij.ui.dsl.builder.*
import com.intellij.ui.layout.ValidationInfoBuilder
import com.intellij.util.asSafely
import com.intellij.util.download.DownloadableFileSetVersions
import org.jetbrains.idea.maven.model.MavenId
import org.jetbrains.idea.maven.wizards.MavenNewProjectWizardStep
import org.jetbrains.plugins.groovy.GroovyBundle
import org.jetbrains.plugins.groovy.config.loadLatestGroovyVersions
import org.jetbrains.plugins.groovy.config.wizard.*
import java.awt.Component
import javax.swing.ComboBoxModel
import javax.swing.DefaultListCellRenderer
import javax.swing.JList
import javax.swing.SwingUtilities

class MavenGroovyNewProjectWizard : BuildSystemGroovyNewProjectWizard {
  override val name = MAVEN

  override val ordinal = 100

  override fun createStep(parent: GroovyNewProjectWizard.Step) = Step(parent).chain(::AssetsStep)

  class Step(parent: GroovyNewProjectWizard.Step) :
    MavenNewProjectWizardStep<GroovyNewProjectWizard.Step>(parent),
    BuildSystemGroovyNewProjectWizardData by parent {

    private val addSampleCodeProperty = propertyGraph.property(true)
      .bindBooleanStorage("NewProjectWizard.addSampleCodeState")

    private var addSampleCode by addSampleCodeProperty

    override fun setupSettingsUI(builder: Panel) {
      super.setupSettingsUI(builder)
      with(builder) {
        row(GroovyBundle.message("label.groovy.sdk")) {
          mavenGroovySdkComboBox(groovySdkProperty)
        }.bottomGap(BottomGap.SMALL)
        row {
          checkBox(UIBundle.message("label.project.wizard.new.project.add.sample.code"))
            .bindSelected(addSampleCodeProperty)
            .whenStateChangedFromUi { logAddSampleCodeChanged(it) }
        }.topGap(TopGap.SMALL)
      }
    }

    override fun setupProject(project: Project) {
      super.setupProject(project)

      val groovySdkVersion = groovySdk.getVersion() ?: GROOVY_SDK_FALLBACK_VERSION
      val builder = MavenGroovyNewProjectBuilder(groovySdkVersion).apply {
        moduleJdk = sdk
        name = parentStep.name
        contentEntryPath = "${parentStep.path}/${parentStep.name}"

        isCreatingNewProject = context.isCreatingNewProject

        parentProject = parentData
        aggregatorProject = parentData
        projectId = MavenId(groupId, artifactId, version)
        isInheritGroupId = parentData?.mavenId?.groupId == groupId
        isInheritVersion = parentData?.mavenId?.version == version
        createSampleCode = addSampleCode
      }
      builder.commit(project)
    }

    private fun Row.mavenGroovySdkComboBox(property: ObservableMutableProperty<DistributionInfo?>) {
      comboBox(getInitializedModel(), fallbackAwareRenderer)
        .columns(COLUMNS_MEDIUM)
        .bindItem(property)
        .validationOnInput { validateGroovySdk(property.get()) }
        .whenItemSelectedFromUi { logGroovySdkChanged(context, property.get()) }
    }

    private fun ValidationInfoBuilder.validateGroovySdk(sdk: DistributionInfo?): ValidationInfo? {
      if (sdk == null) {
        return warning(GroovyBundle.message("new.project.wizard.groovy.retrieving.has.failed"))
      }
      return null
    }

    private val fallbackAwareRenderer: DefaultListCellRenderer = object : DefaultListCellRenderer() {
      override fun getListCellRendererComponent(list: JList<*>?,
                                                value: Any?,
                                                index: Int,
                                                isSelected: Boolean,
                                                cellHasFocus: Boolean): Component {
        val representation = value.asSafely<DistributionInfo>()?.getVersion() ?: GROOVY_SDK_FALLBACK_VERSION // NON-NLS
        return super.getListCellRendererComponent(list, representation, index, isSelected, cellHasFocus)
      }
    }

    private fun getInitializedModel(): ComboBoxModel<DistributionInfo?> {
      val model = CollectionComboBoxModel<DistributionInfo?>()
      loadLatestGroovyVersions(object : DownloadableFileSetVersions.FileSetVersionsCallback<FrameworkLibraryVersion>() {
        override fun onSuccess(versions: MutableList<out FrameworkLibraryVersion>) {
          SwingUtilities.invokeLater {
            for (version in versions.sortedWith(::moveUnstableVersionToTheEnd)) {
              model.add(FrameworkLibraryDistributionInfo(version))
            }
            model.selectedItem = model.items.first()
          }
        }

        override fun onError(errorMessage: String) {
          model.add(null)
          model.selectedItem = model.items.first()
        }
      })
      return model
    }
  }

  private class AssetsStep(parent: NewProjectWizardStep) : AssetsNewProjectWizardStep(parent) {
    override fun setupAssets(project: Project) {
      outputDirectory = "$path/$name"
      if (gitData?.git == true) {
        addAssets(StandardAssetsProvider().getMavenIgnoreAssets())
      }
    }
  }
}

private fun DistributionInfo?.getVersion() = when(this) {
  is FrameworkLibraryDistributionInfo -> this.version.versionString
  null -> null
  else -> {
    logger<MavenGroovyNewProjectWizard>().error("Unexpected distribution type")
    null
  }
}

==> maven-231.8770.65-sources.jar: org/jetbrains/idea/maven/plugins/groovy/wizard/MavenGroovyNewProjectWizard.kt <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package org.jetbrains.idea.maven.plugins.groovy.wizard

import com.intellij.framework.library.FrameworkLibraryVersion
import com.intellij.ide.projectWizard.NewProjectWizardCollector.Base.logAddSampleCodeChanged
import com.intellij.ide.projectWizard.NewProjectWizardConstants.BuildSystem.MAVEN
import com.intellij.ide.projectWizard.generators.AssetsNewProjectWizardStep
import com.intellij.ide.starters.local.StandardAssetsProvider
import com.intellij.ide.wizard.NewProjectWizardChainStep.Companion.nextStep
import com.intellij.ide.wizard.NewProjectWizardStep
import com.intellij.ide.wizard.NewProjectWizardStep.Companion.ADD_SAMPLE_CODE_PROPERTY_NAME
import com.intellij.ide.wizard.setupProjectFromBuilder
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.observable.util.bindBooleanStorage
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.ui.distribution.DistributionInfo
import com.intellij.openapi.ui.ValidationInfo
import com.intellij.ui.CollectionComboBoxModel
import com.intellij.ui.UIBundle
import com.intellij.ui.dsl.builder.*
import com.intellij.ui.layout.ValidationInfoBuilder
import com.intellij.util.asSafely
import com.intellij.util.download.DownloadableFileSetVersions
import org.jetbrains.idea.maven.model.MavenId
import org.jetbrains.idea.maven.wizards.MavenNewProjectWizardStep
import org.jetbrains.plugins.groovy.GroovyBundle
import org.jetbrains.plugins.groovy.config.loadLatestGroovyVersions
import org.jetbrains.plugins.groovy.config.wizard.*
import java.awt.Component
import javax.swing.ComboBoxModel
import javax.swing.DefaultListCellRenderer
import javax.swing.JList
import javax.swing.SwingUtilities

class MavenGroovyNewProjectWizard : BuildSystemGroovyNewProjectWizard {
  override val name = MAVEN

  override val ordinal = 100

  override fun createStep(parent: GroovyNewProjectWizard.Step): NewProjectWizardStep =
    Step(parent)
      .nextStep(::AssetsStep)

  class Step(parent: GroovyNewProjectWizard.Step) :
    MavenNewProjectWizardStep<GroovyNewProjectWizard.Step>(parent),
    BuildSystemGroovyNewProjectWizardData by parent {

    private val addSampleCodeProperty = propertyGraph.property(true)
      .bindBooleanStorage(ADD_SAMPLE_CODE_PROPERTY_NAME)

    private var addSampleCode by addSampleCodeProperty

    private fun setupGroovySdkUI(builder: Panel) {
      builder.row(GroovyBundle.message("label.groovy.sdk")) {
        comboBox(getInitializedModel(), fallbackAwareRenderer)
          .columns(COLUMNS_MEDIUM)
          .bindItem(groovySdkProperty)
          .validationOnInput { validateGroovySdk(groovySdk) }
          .whenItemSelectedFromUi { logGroovySdkChanged(groovySdk) }
      }.bottomGap(BottomGap.SMALL)
    }

    private fun setupSampleCodeUI(builder: Panel) {
      builder.row {
        checkBox(UIBundle.message("label.project.wizard.new.project.add.sample.code"))
          .bindSelected(addSampleCodeProperty)
          .whenStateChangedFromUi { logAddSampleCodeChanged(it) }
      }
    }

    override fun setupSettingsUI(builder: Panel) {
      setupJavaSdkUI(builder)
      setupGroovySdkUI(builder)
      setupParentsUI(builder)
      setupSampleCodeUI(builder)
    }

    override fun setupAdvancedSettingsUI(builder: Panel) {
      setupGroupIdUI(builder)
      setupArtifactIdUI(builder)
    }

    override fun setupProject(project: Project) {
      val groovySdkVersion = groovySdk.getVersion() ?: GROOVY_SDK_FALLBACK_VERSION
      val builder = MavenGroovyNewProjectBuilder(groovySdkVersion).apply {
        moduleJdk = sdk
        name = parentStep.name
        contentEntryPath = "${parentStep.path}/${parentStep.name}"

        isCreatingNewProject = context.isCreatingNewProject

        parentProject = parentData
        aggregatorProject = parentData
        projectId = MavenId(groupId, artifactId, version)
        isInheritGroupId = parentData?.mavenId?.groupId == groupId
        isInheritVersion = parentData?.mavenId?.version == version
        createSampleCode = addSampleCode
      }
      setupProjectFromBuilder(project, builder)
    }

    private fun ValidationInfoBuilder.validateGroovySdk(sdk: DistributionInfo?): ValidationInfo? {
      if (sdk == null) {
        return warning(GroovyBundle.message("new.project.wizard.groovy.retrieving.has.failed"))
      }
      return null
    }

    private val fallbackAwareRenderer: DefaultListCellRenderer = object : DefaultListCellRenderer() {
      override fun getListCellRendererComponent(list: JList<*>?,
                                                value: Any?,
                                                index: Int,
                                                isSelected: Boolean,
                                                cellHasFocus: Boolean): Component {
        val representation = value.asSafely<DistributionInfo>()?.getVersion() ?: GROOVY_SDK_FALLBACK_VERSION // NON-NLS
        return super.getListCellRendererComponent(list, representation, index, isSelected, cellHasFocus)
      }
    }

    private fun getInitializedModel(): ComboBoxModel<DistributionInfo?> {
      val model = CollectionComboBoxModel<DistributionInfo?>()
      loadLatestGroovyVersions(object : DownloadableFileSetVersions.FileSetVersionsCallback<FrameworkLibraryVersion>() {
        override fun onSuccess(versions: List<FrameworkLibraryVersion>) {
          SwingUtilities.invokeLater {
            for (version in versions.sortedWith(::moveUnstableVersionToTheEnd)) {
              model.add(FrameworkLibraryDistributionInfo(version))
            }
            model.selectedItem = model.items.first()
          }
        }

        override fun onError(errorMessage: String) {
          model.add(null)
          model.selectedItem = model.items.first()
        }
      })
      return model
    }
  }

  private class AssetsStep(parent: NewProjectWizardStep) : AssetsNewProjectWizardStep(parent) {

    override fun setupAssets(project: Project) {
      addAssets(StandardAssetsProvider().getMavenIgnoreAssets())
    }
  }
}

private fun DistributionInfo?.getVersion() = when(this) {
  is FrameworkLibraryDistributionInfo -> this.version.versionString
  null -> null
  else -> {
    logger<MavenGroovyNewProjectWizard>().error("Unexpected distribution type")
    null
  }
}
